from pathlib import Path
from typing import Type, TYPE_CHECKING, TypeVar
from django.core.exceptions import PermissionDenied
from django.db import models
from django.conf import settings


class ModelManager(models.Manager):
    def try_get(self, **kwargs):
        qs = self.get_queryset().filter(**kwargs)
        if qs.exists():
            return qs.get()
        else:
            return None


def get_list(v):
    if isinstance(v, list):
        return v
    else:
        return [v]


def compact(l):
    return [v for v in l if v]


def check_permission(request):
    must_login = getattr(settings, "FILE_FORM_MUST_LOGIN", False)

    if must_login and not request.user.is_authenticated:
        raise PermissionDenied()


def get_upload_path():
    default_upload_dir = "temp_uploads"
    upload_path = Path(getattr(settings, "FILE_FORM_UPLOAD_DIR", default_upload_dir))

    if upload_path.is_absolute():
        return upload_path
    else:
        return Path(settings.MEDIA_ROOT).joinpath(upload_path)


T = TypeVar("T")


def with_typehint(baseclass: Type[T]) -> Type[T]:
    """
    Useful function to make mixins with baseclass typehint

    ```
    class ReadonlyMixin(with_typehint(BaseAdmin))):
        ...
    ```
    """
    if TYPE_CHECKING:
        return baseclass

    return object



# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2017-03-16 14:01
from django.db import migrations, models

from . import storage


class Migration(migrations.Migration):

    dependencies = [
        ("django_file_form", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="uploadedfile",
            name="uploaded_file",
            field=models.FileField(
                max_length=255, storage=storage, upload_to="temp_uploads"
            ),
        ),
    ]



import json
from typing import Dict, Union
from django.forms import ClearableFileInput
from django.http import QueryDict
from django.utils.datastructures import MultiValueDict
from django.utils.translation import gettext as _
from django_file_form.util import compact

from .uploaded_file import PlaceholderUploadedFile, S3UploadedFileWithId


TRANSLATIONS = {
    "Cancel": _("Cancel"),
    "Delete": _("Delete"),
    "Delete failed": _("Delete failed"),
    "Upload failed": _("Upload failed"),
    "Drop your files here": _("Drop your files here"),
}


def get_upload(upload_data: Dict):
    upload_type = upload_data["type"]

    if upload_type == "placeholder":
        return PlaceholderUploadedFile(
            file_id=upload_data["id"],
            name=upload_data["name"],
            size=upload_data["size"],
        )
    elif upload_type == "s3":
        return S3UploadedFileWithId(
            file_id=upload_data["id"],
            name=upload_data["name"],
            original_name=upload_data["original_name"],
            size=upload_data["size"],
        )
    else:
        return None


def get_uploads(data: QueryDict, prefixed_field_name: str):
    placeholder_field_name = prefixed_field_name + "-uploads"
    value = data.get(placeholder_field_name)

    if not value:
        return []
    else:
        return compact([get_upload(upload_data) for upload_data in json.loads(value)])


def get_file_meta(data: QueryDict, prefixed_field_name: str):
    meta_field_name = prefixed_field_name + "-metadata"
    value = data.get(meta_field_name)
    if not value:
        return {}
    try:
        res = json.loads(value)
        if not isinstance(res, dict):
            return {}
        return res
    except Exception:
        return {}


class BaseUploadWidget(ClearableFileInput):
    template_name = "django_file_form/upload_widget.html"

    def get_context(self, name, value, attrs):
        context = super().get_context(name, value, attrs)

        context["translations"] = json.dumps(TRANSLATIONS)

        return context


class UploadWidget(BaseUploadWidget):
    def value_from_datadict(
        self, data: QueryDict, files: MultiValueDict, prefixed_field_name: str
    ):
        upload = super().value_from_datadict(data, files, prefixed_field_name)

        if upload:
            return upload
        else:
            uploads = get_uploads(data, prefixed_field_name)

            upload = uploads[0] if uploads else None
            metadata = get_file_meta(data, prefixed_field_name)

            if upload and upload.name in metadata:
                upload.metadata = metadata[upload.name]
            return upload


class UploadMultipleWidget(BaseUploadWidget):
    def value_from_datadict(
        self,
        data: QueryDict,
        files: Union[Dict, MultiValueDict],
        prefixed_field_name: str,
    ):
        def get_uploads_from_files():
            if hasattr(files, "getlist"):
                return files.getlist(prefixed_field_name)
            else:
                # NB: django-formtools wizard uses dict instead of MultiValueDict
                return (
                    super(UploadMultipleWidget, self).value_from_datadict(
                        data, files, prefixed_field_name
                    )
                    or []
                )

        uploads = get_uploads_from_files() + get_uploads(data, prefixed_field_name)
        metadata = get_file_meta(data, prefixed_field_name)

        for upload in uploads:
            if upload.name in metadata:
                upload.metadata = metadata[upload.name]
        return uploads



import os
from pathlib import Path

from django.conf import settings
from django.core.files.storage import FileSystemStorage
from django.db import models
from django.utils import timezone
from django.utils.module_loading import import_string

from .util import ModelManager, get_upload_path

# Import uploaded files for backward compatibility
from .uploaded_file import (
    PlaceholderUploadedFile,
    S3UploadedFileWithId,
    UploadedTusFile,
)


class TemporaryUploadedFileManager(ModelManager):
    def delete_unused_files(self, delete=True, now=None):
        deleted_files = []

        for t in self.get_queryset():
            if t.must_be_deleted(now):
                deleted_files.append(Path(t.uploaded_file.name).name)

                if delete:
                    t.delete()

        temp_path = get_upload_path()

        for f in temp_path.iterdir():
            basename = f.name

            if not self.get_for_file(basename):
                deleted_files.append(basename)

                if delete:
                    f.unlink()

        return deleted_files

    def for_field_and_form(self, field_name, form_id):
        return self.filter(form_id=form_id, field_name=field_name)

    def get_for_file(self, filename):
        return self.try_get(uploaded_file=get_upload_to_for_filename(filename))


def get_upload_to_for_filename(filename):
    path = getattr(settings, "FILE_FORM_UPLOAD_DIR", "temp_uploads")
    return os.path.join(path, filename)


def get_upload_to(_instance, filename):
    return get_upload_to_for_filename(filename)


if hasattr(settings, "FILE_FORM_TEMP_STORAGE"):
    storage_class = import_string(settings.FILE_FORM_TEMP_STORAGE)
else:
    storage_class = FileSystemStorage


class TemporaryUploadedFile(models.Model):
    """
    TemporaryUploadedFile is used for temporary storage of an uploaded file.

    * Added when a file is uploaded using the tus ajax upload
    * Removed when the form is submitted sucessfully
    * Or removed later by 'python manage.py delete_unused_files'
    """

    created = models.DateTimeField(default=timezone.now)
    uploaded_file = models.FileField(
        max_length=255, storage=storage_class(), upload_to=get_upload_to
    )
    original_filename = models.CharField(max_length=255)
    field_name = models.CharField(max_length=255, null=True, blank=True)
    file_id = models.CharField(max_length=40)
    form_id = models.CharField(max_length=40)

    objects = TemporaryUploadedFileManager()

    class Meta(object):
        index_together = (("form_id", "field_name"),)
        db_table = "django_file_form_uploadedfile"

    def __str__(self):
        return str(self.original_filename or "")

    def delete(self, *args, **kwargs):
        if self.uploaded_file and self.uploaded_file.storage.exists(
            self.uploaded_file.name
        ):
            self.uploaded_file.delete()

        super().delete(*args, **kwargs)

    def get_uploaded_file(self):
        return UploadedTusFile(
            file=self.uploaded_file, name=self.original_filename, file_id=self.file_id
        )

    def must_be_deleted(self, now=None):
        now = now or timezone.now()

        return (now - self.created).days >= 1



from django.urls import path
from .views import handle_upload, start_upload


urlpatterns = [
    path("", start_upload, name="tus_upload"),
    path("<str:resource_id>", handle_upload, name="tus_upload_chunks"),
]



# Generated by Django 3.0.4 on 2020-04-07 13:14

from django.db import migrations, models

from . import storage


class Migration(migrations.Migration):

    dependencies = [
        ("django_file_form", "0004_auto_20170423_0329"),
    ]

    operations = [
        migrations.AlterField(
            model_name="uploadedfile",
            name="uploaded_file",
            field=models.FileField(
                max_length=255, storage=storage, upload_to="temp_uploads"
            ),
        ),
    ]



from django.conf import settings
from django.core.files.storage import FileSystemStorage
from django.utils.module_loading import import_string


if hasattr(settings, "FILE_FORM_TEMP_STORAGE"):
    storage_class = import_string(settings.FILE_FORM_TEMP_STORAGE)
else:
    storage_class = FileSystemStorage

storage = storage_class()



import os
import time

import boto3
from botocore.exceptions import ClientError
from botocore.client import Config
from django.utils.crypto import get_random_string
from storages.utils import setting, lookup_env


def get_bucket_name():
    return setting("AWS_STORAGE_BUCKET_NAME") or lookup_env(
        ["DJANGO_AWS_STORAGE_BUCKET_NAME"]
    )


def get_access_key_id():
    return setting("AWS_S3_ACCESS_KEY_ID", setting("AWS_ACCESS_KEY_ID")) or lookup_env(
        ["AWS_S3_ACCESS_KEY_ID", "AWS_ACCESS_KEY_ID"]
    )


def get_secret_access_key():
    return setting(
        "AWS_S3_SECRET_ACCESS_KEY", setting("AWS_SECRET_ACCESS_KEY")
    ) or lookup_env(["AWS_S3_SECRET_ACCESS_KEY", "AWS_SECRET_ACCESS_KEY"])


def get_endpoint_url():
    return setting("AWS_S3_ENDPOINT_URL") or lookup_env(
        ["AWS_S3_ENDPOINT_URL", "AWS_ENDPOINT_URL"]
    )


def file_form_upload_dir():
    return setting("FILE_FORM_UPLOAD_DIR", "temp_uploads")


def get_client():
    signature_version = setting("AWS_S3_SIGNATURE_VERSION", None)
    region_name = setting("AWS_S3_REGION_NAME", None)

    while True:
        try:
            # https://github.com/boto/boto3/issues/801
            return boto3.client(
                "s3",
                endpoint_url=get_endpoint_url(),
                aws_access_key_id=get_access_key_id(),
                aws_secret_access_key=get_secret_access_key(),
                config=Config(
                    signature_version=signature_version, region_name=region_name
                ),
            )
        except:
            time.sleep(0.01)


def exists(client, bucket_name, name):
    """
    Check if key already exists in bucket.

    Code adapted from storage.backends.s3boto3
    """
    try:
        client.head_object(Bucket=bucket_name, Key=name)
        return True
    except ClientError:
        return False


def get_alternative_name(file_root, file_ext):
    """
    Return an alternative filename, by adding an underscore and a random 7
    character alphanumeric string (before the file extension, if one
    exists) to the filename.

    Code adapted from django.storage.get_alternative_name
    """
    return f"{file_root}_{get_random_string(7)}{file_ext}"


def get_available_name(client, bucket_name, name):
    """
    Return a filename that's free on the target storage system and
    available for new content to be written to.

    Code adapted from django.storage.get_available_name
    """
    dir_name, file_name = os.path.split(name)
    file_root, file_ext = os.path.splitext(file_name)
    # If the filename already exists, generate an alternative filename
    # until it doesn't exist.
    while exists(client, bucket_name, name):
        # file_ext includes the dot.
        name = os.path.join(dir_name, get_alternative_name(file_root, file_ext))

    return name



from django.db import models
from django.forms import ModelForm
from django.contrib import admin

from .fields import UploadedFileField
from .forms import FileFormMixin
from .util import with_typehint
from .widgets import UploadWidget


class FileAdminForm(FileFormMixin, ModelForm):
    pass


class FileFormAdminMixin(with_typehint(admin.ModelAdmin)):
    change_form_template = "django_file_form/admin_change_form.html"
    form = FileAdminForm
    formfield_overrides = {
        models.FileField: {
            "form_class": UploadedFileField,
            "widget": UploadWidget,
        }
    }

    class Media:
        css = {"all": ("file_form/file_form.css",)}
        js = ("file_form/file_form.js", "file_form/auto_init.js")

    def save_model(self, request, obj, form, change):
        super().save_model(request, obj, form, change)

        form.delete_temporary_files()


class FileFormAdmin(FileFormAdminMixin, admin.ModelAdmin):
    pass



from django.contrib import admin

from .models import TemporaryUploadedFile


class TemporaryUploadedFileAdmin(admin.ModelAdmin):
    list_display = ["original_filename", "created"]
    date_hierarchy = "created"


admin.site.register(TemporaryUploadedFile, TemporaryUploadedFileAdmin)



from django.urls import path


try:
    from . import views

    urlpatterns = [
        path("", views.create_upload, name="s3_upload"),
        path(
            "<upload_id>/",
            views.abort_upload,
            name="get_parts_or_abort_upload",
        ),
        path(
            "<upload_id>/<int:part_number>",
            views.sign_upload_part,
            name="sign_part_upload",
        ),
        path(
            "<upload_id>/complete",
            views.complete_upload,
            name="complete_multipart_upload",
        ),
    ]
except ImportError:  # pragma: no cover
    urlpatterns = []
